/***
  * The contents of this file are subject to the Mozilla Public
  * License Version 1.1 (the "License"); you may not use this file
  * except in compliance with the License. You may obtain a copy of
  * the License at http://www.mozilla.org/MPL/
  * 
  * Software distributed under the License is distributed on an "AS
  * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
  * implied. See the License for the specific language governing
  * rights and limitations under the License.
  * 
  * The Original Code in this file as it was released publicly upon
  * October 30, 2004 after being tested for several months by a 
  * bunch of CSS co-workers. Thanks guys ;)
  * 
  * The Initial Developer of the Original Code is HJ van Rantwijk.
  * all Portions created by HJ van Rantwijk are Copyright (C) 2004-2006
  * by HJ van Rantwijk. All Rights Reserved.
  * 
  * Contributors:
  *   Michael Vincent van Rantwijk <mv_van_rantwijk@yahoo.com>
  *
  */

const DC_INFO_ARRAY   = 0;
const DC_TYPE         = 0;
const DC_URL          = 1;
const DC_FAVICON      = 2;
const DC_TITLE        = 3;
const DC_LINK         = 4;
const DC_DESCRIPTION  = 5;
const DC_IMAGE        = 6;
const DC_LASTMODIFIED = 7;

const NO_ERROR        = 0;
const DUPLICATE_ERROR = 1;
const NO_FEED_ERROR   = 2;
const DOC_TYPE_ERROR  = 3;
const DOC_EMPTY_ERROR = 4;

const DC_IMAGE_URL    = 0;
const DC_IMAGE_TITLE  = 1;
const DC_IMAGE_WIDTH  = 2;
const DC_IMAGE_HEIGHT = 3;

var gNavigatorWindow, gEntryField, gErrorCode, gAcceptState = false;
var gDataCollection, gAcceptButton, gImageStat, gBundle;

function selectWindowStyling(aStyleFlag)
{
  var _documentElement = document.documentElement;

  if (aStyleFlag)
    _documentElement.removeAttribute("extraStyling");
  else
    _documentElement.setAttribute("extraStyling", "true");
}

function initDialog()
{
  selectWindowStyling(window.arguments[3]);
  gEntryField = document.getElementById("EntryField");
  var url = window.arguments[2];

  // gEntryField.value = (url && url.match(/^file:/)) ? url : (url && url.match(/^https?:\/\/\w.\w/)) ? url : "";
  gEntryField.value = (url && url.match(/^https?:\/\/\w.\w/)) ? url : "";
  gNavigatorWindow = window.arguments[1];
  gAcceptButton = document.documentElement.getButton('accept');
  gBundle = document.getElementById("multizilla-properties");
  onTextboxChange();
}

function getFeedInfo(aURL)
{
  var tabbrowser = gNavigatorWindow.getBrowser();
  var xmlRequest = new XMLHttpRequest();
  var handler = this;

  document.documentElement.getButton('accept').setAttribute("image", "chrome://multiviews/content/images/feedpanel/loading.gif");
  gErrorCode = 0;

  xmlRequest.open("GET", aURL);
  xmlRequest.setRequestHeader("User-Agent", "The MultiZilla Feed Reader/Viewer");
  xmlRequest.overrideMimeType("application/xml");
  xmlRequest.onerror = function(event) { onXMLError(event); };
  xmlRequest.onload = function()
  {
    gErrorCode = 0;

    if (xmlRequest.readyState == 4) {
      var channel = xmlRequest.channel.QueryInterface(Components.interfaces.nsIHttpChannel);
      var xmldoc = (channel && channel.responseStatus == 200) ? xmlRequest.responseXML : null;

      if (xmldoc) { 
        if (!(xmldoc instanceof XMLDocument && xmldoc instanceof XULDocument)) {
          if (gNavigatorWindow.mzFeedHandler.isDocumentFeed(xmldoc)) {
            if (gNavigatorWindow.mzFeedHandler.isNewFeedURL(aURL)) {
              var dataCollection = gNavigatorWindow.mzFeedHandler.getFeedInfoFromDocument(xmldoc);
              dataCollection[DC_URL] = aURL;
              var _nsIIOService = Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService);
              var uri = _nsIIOService.newURI(aURL, null, null);
              dataCollection[DC_FAVICON] = tabbrowser.buildFavIconString(uri);
              dataCollection = gNavigatorWindow.mzFeedHandler.getDataCollectionFromDocument(xmldoc, dataCollection);
              displayFeedInfo(dataCollection);
              return(NO_ERROR);
            }
            return displayErrorMessage(DUPLICATE_ERROR);
          }
          return displayErrorMessage(NO_FEED_ERROR);
        }
        return displayErrorMessage(DOC_TYPE_ERROR);
      }
      return displayErrorMessage(DOC_EMPTY_ERROR);
    }
    return(NO_ERROR);
  }
  xmlRequest.send(null);
}

function onXMLError(aEvent)
{
  var request, status;

  try {
    request = aEvent.target;
    status = request.status;
  }
  catch(ex) {
    request = aEvent.target.channel.QueryInterface(Components.interfaces.nsIRequest);
    status = request.status;
  }
  if (status > 2152398848) // see: http://lxr.mozilla.org/seamonkey/source/netwerk/base/public/nsNetError.h
    status -= status;
    displayErrorMessage(status);
}

function displayErrorMessage(aErrorCode)
{
  if (gErrorCode >= 0) {
    gPreviousErrorCode = aErrorCode;
    document.documentElement.getButton('accept').removeAttribute("image");
    var promptService = Components.classes["@mozilla.org/embedcomp/prompt-service;1"].getService(Components.interfaces.nsIPromptService);
    var title = gBundle.getString("addFeedWindowAlertTitle");
    var errorHeader = gBundle.getString("addFeedWindowError");
    var errors = gBundle.getString("addFeedWindowErrorText");
    errors = errors.split(',');
    var text = gBundle.getString("addFeedWindowAlertText");
    text = text + errorHeader + " " + errors[aErrorCode] + "\n";
    promptService.alert(window, title, text);
  }
}

function displayFeedInfo(aDataCollection)
{
  /* Images take time to load so let's do those first */
  var feedImageURL = aDataCollection[DC_INFO_ARRAY][DC_IMAGE][DC_IMAGE_URL];

  if (feedImageURL)
    document.getElementById("FeedImage-dummy").setAttribute("src", feedImageURL);

  var feedFaviconURL = aDataCollection[DC_INFO_ARRAY][DC_FAVICON];

  if (feedFaviconURL)
    document.getElementById("FeedImage-dummyFavicon").setAttribute("src", feedFaviconURL);

  var element;
  var mzRDF = "http://multizilla.mozdev.org/rdf#";
  var elementIDArray = [ "Type", "Title", "Url", "Description", "LastModified" ];
  var attributeArray = [ "value", "value", "value", "value", "value" ];
  var dataIndexArray = [ DC_TYPE, DC_TITLE, DC_URL, DC_DESCRIPTION, DC_LASTMODIFIED ];

  for (var i = 0; i < 5; i++) {
    element = document.getElementById(elementIDArray[i]);
    element.setAttribute(attributeArray[i], aDataCollection[DC_INFO_ARRAY][dataIndexArray[i]]);
  }
  document.getElementById("ItemCounter").value = aDataCollection.length;
  var lastModified = "No Date Specified";
  var lastModifiedDate = aDataCollection[DC_INFO_ARRAY][DC_LASTMODIFIED];

  if (lastModifiedDate) {
    lastModified = new Date(aDataCollection[DC_INFO_ARRAY][DC_LASTMODIFIED]).toUTCString();
  }
  document.getElementById("LastModified").value = lastModified;

  if (feedImageURL)
    document.getElementById("FeedImageBox").removeAttribute("collapsed");

  document.getElementById("EntryFieldBox").setAttribute("collapsed", "true");
  document.getElementById("FeedInfoBox").removeAttribute("collapsed");
  document.title = document.getElementById("FeedInfoTitle").getAttribute("title");
  gAcceptButton.removeAttribute("image");
  gAcceptButton.setAttribute("type", "add");
  gAcceptButton.label = " Add Feed"; // XXX: Localize me!!!
  sizeToContent();
  gDataCollection = aDataCollection;
  gAcceptState = true;
}

function onTextboxChange()
{
  var disabled = true;
  var url = gEntryField.value

  /* if (url && url.match(/^https?:\/\/\w.\w/)) */
  if (isValidURL(url))
    gAcceptButton.removeAttribute("disabled");
  else 
    gAcceptButton.setAttribute("disabled", "true");
}

function onCancel()
{
  gNavigatorWindow.mzFeedHandler._newFeedAdded = false;
}

function onAccept()
{
  gNavigatorWindow.mzFeedHandler._newFeedAdded = true;

  if (!gAcceptState) {
    var url = gEntryField.value;

    /* if (url && url.match(/^https?:\/\/\w.\w/)) */
    if (isValidURL(url))
      getFeedInfo(url);
  }
  else {
    gDataCollection[DC_INFO_ARRAY][DC_TITLE] = document.getElementById("Title").value;
    gDataCollection[DC_INFO_ARRAY][DC_DESCRIPTION] = document.getElementById("Description").value;

    gDataCollection[0][9] = new Date().getTime();
    var dummy = gNavigatorWindow.mzFeedHandler.addFeedFromDataCollection(gDataCollection, false);
    gAcceptState = true;
  }
  return gAcceptState;
}

function onImageLoad(aElement)
{
  var element, imageURL = aElement.getAttribute("src");
  dump("\nonImageLoad() " + aElement.id);

  if (aElement.id == "FeedImage-dummy") {
    element = document.getElementById("FeedImage");
    element.setAttribute("src", imageURL);

    var width = gDataCollection[DC_INFO_ARRAY][DC_IMAGE][DC_IMAGE_WIDTH];
    var height = gDataCollection[DC_INFO_ARRAY][DC_IMAGE][DC_IMAGE_HEIGHT];

    if (width)
      element.setAttribute("width", width);
    if (height)
      element.setAttribute("height", height);
  }
  if (aElement.id == "FeedImage-dummyFavicon")
    document.getElementById("FeedImageFavicon").setAttribute("src", imageURL);
}

function isValidURL(aURL)
{
  if (/* aURL && aURL.match(/^file:/) || */ aURL && aURL.match(/^https?:\/\/\w.\w/))
    return true;
  return false;
}
